@@ -1,6 +1,6 @@
 /*
 linphone
-Copyright (C) 2000  Simon MORLAT (simon.morlat@linphone.org)
+Copyright (C) 2000 - 2010 Simon MORLAT (simon.morlat@linphone.org)
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -127,6 +127,8 @@ typedef struct _LinphoneCallLog{
 	int duration; /**<Duration of the call in seconds*/
 	char *refkey;
 	void *user_pointer;
+	rtp_stats_t local_stats;
+	rtp_stats_t remote_stats;
 	struct _LinphoneCore *lc;
 } LinphoneCallLog;
 
@@ -137,6 +139,8 @@ void linphone_call_log_set_user_pointer(LinphoneCallLog *cl, void *up);
 void *linphone_call_log_get_user_pointer(const LinphoneCallLog *cl);
 void linphone_call_log_set_ref_key(LinphoneCallLog *cl, const char *refkey);
 const char *linphone_call_log_get_ref_key(const LinphoneCallLog *cl);
+const rtp_stats_t *linphone_call_log_get_local_stats(const LinphoneCallLog *cl);
+const rtp_stats_t *linphone_call_log_get_remote_stats(const LinphoneCallLog *cl);
 char * linphone_call_log_to_str(LinphoneCallLog *cl);
 
 typedef enum{
@@ -345,6 +349,7 @@ typedef enum _gstate {
   GSTATE_REG_NONE = 10,       /* initial state */
   GSTATE_REG_OK,
   GSTATE_REG_FAILED,
+  GSTATE_REG_PENDING, /* a registration request is ongoing*/
   /* states for GSTATE_GROUP_CALL */
   GSTATE_CALL_IDLE = 20,      /* initial state */
   GSTATE_CALL_OUT_INVITE,
@@ -353,7 +358,8 @@ typedef enum _gstate {
   GSTATE_CALL_IN_CONNECTED,
   GSTATE_CALL_END,
   GSTATE_CALL_ERROR,
-  GSTATE_INVALID
+  GSTATE_INVALID,
+  GSTATE_CALL_OUT_RINGING /*remote ringing*/
 } gstate_t;
 
 struct _LinphoneGeneralState {
@@ -785,6 +791,8 @@ void linphone_core_destroy(LinphoneCore *lc);
 /*for advanced users:*/
 void linphone_core_set_audio_transports(LinphoneCore *lc, RtpTransport *rtp, RtpTransport *rtcp);
 
+int linphone_core_get_current_call_stats(LinphoneCore *lc, rtp_stats_t *local, rtp_stats_t *remote);
+
 #ifdef __cplusplus
 }
 #endif
