@@ -22,16 +22,29 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "mediastreamer2/mediastream.h"
 #include <stdlib.h>
 #include <stdio.h>
+#ifdef HAVE_SIGHANDLER_T
 #include <signal.h>
+#endif /*HAVE_SIGHANDLER_T*/
+
+#include <string.h>
+#if !defined(_WIN32_WCE)
+#include <errno.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
 #include <fcntl.h>
+#endif /*_WIN32_WCE*/
 
 #undef snprintf
 #include <ortp/stun.h>
 
-#ifndef WIN32
+#ifdef HAVE_GETIFADDRS
+#include <net/if.h>
+#include <ifaddrs.h>
+#endif
+
+
+#if !defined(WIN32)
 
 static char lock_name[80];
 static char lock_set=0;
@@ -39,7 +52,7 @@ static char lock_set=0;
 int set_lock_file()
 {
 	FILE *lockfile;
-	
+
 	snprintf(lock_name,80,"/tmp/linphone.%i",getuid());
 	lockfile=fopen(lock_name,"w");
 	if (lockfile==NULL)
@@ -58,7 +71,7 @@ int get_lock_file()
 {
 	int pid;
 	FILE *lockfile;
-	
+
 	snprintf(lock_name,80,"/tmp/linphone.%i",getuid());
 	lockfile=fopen(lock_name,"r");
 	if (lockfile==NULL)
@@ -95,15 +108,17 @@ char *int2str(int number)
 
 void check_sound_device(LinphoneCore *lc)
 {
-	int fd,len;
+#ifdef _linux
+	int fd=0;
+	int len;
 	int a;
 	char *file=NULL;
 	char *i810_audio=NULL;
 	char *snd_pcm_oss=NULL;
 	char *snd_mixer_oss=NULL;
 	char *snd_pcm=NULL;
-	
 	fd=open("/proc/modules",O_RDONLY);
+
 	if (fd>0){
 		/* read the entire /proc/modules file and check if sound conf seems correct */
 		/*a=fstat(fd,&statbuf);
@@ -136,13 +151,12 @@ void check_sound_device(LinphoneCore *lc)
 			}
 		}
 	}else {
-#ifdef __linux
+
 		ms_warning("Could not open /proc/modules.");
-#endif
 	}
 	/* now check general volume. Some user forget to rise it and then complain that linphone is
 	not working */
-	/* but some other users complain that linphone should not change levels... 
+	/* but some other users complain that linphone should not change levels...
 	if (lc->sound_conf.sndcard!=NULL){
 		a=snd_card_get_level(lc->sound_conf.sndcard,SND_CARD_LEVEL_GENERAL);
 		if (a<50){
@@ -154,50 +168,73 @@ void check_sound_device(LinphoneCore *lc)
 	end:
 	if (file!=NULL) ms_free(file);
 	if (fd>0) close(fd);
+#endif
 }
 
 #define UDP_HDR_SZ 8
 #define RTP_HDR_SZ 12
 #define IP4_HDR_SZ 20   /*20 is the minimum, but there may be some options*/
 
-const char *payload_type_get_description(PayloadType *pt){
-	return _((const char *)pt->user_data);
-}	
-
-void payload_type_set_enable(PayloadType *pt,int value) 
+static void payload_type_set_enable(PayloadType *pt,int value)
 {
 	if ((value)!=0) payload_type_set_flag(pt,PAYLOAD_TYPE_ENABLED); \
-	else payload_type_unset_flag(pt,PAYLOAD_TYPE_ENABLED); 
+	else payload_type_unset_flag(pt,PAYLOAD_TYPE_ENABLED);
 }
 
-
-bool_t payload_type_enabled(PayloadType *pt) {
+static bool_t payload_type_enabled(PayloadType *pt) {
 	return (((pt)->flags & PAYLOAD_TYPE_ENABLED)!=0);
 }
 
-int payload_type_get_bitrate(PayloadType *pt)
-{
+bool_t linphone_core_payload_type_enabled(LinphoneCore *lc, PayloadType *pt){
+	if (ms_list_find(lc->codecs_conf.audio_codecs,pt) || ms_list_find(lc->codecs_conf.video_codecs,pt)){
+		return payload_type_enabled(pt);
+	}
+	ms_error("Getting enablement status of codec not in audio or video list of PayloadType !");
+	return FALSE;
+}
+
+int linphone_core_enable_payload_type(LinphoneCore *lc, PayloadType *pt, bool_t enabled){
+	if (ms_list_find(lc->codecs_conf.audio_codecs,pt) || ms_list_find(lc->codecs_conf.video_codecs,pt)){
+		payload_type_set_enable(pt,enabled);
+		return 0;
+	}
+	ms_error("Enabling codec not in audio or video list of PayloadType !");
+	return -1;
+}
+
+const char *linphone_core_get_payload_type_description(LinphoneCore *lc, PayloadType *pt){
+	if (ms_filter_codec_supported(pt->mime_type)){
+		MSFilterDesc *desc=ms_filter_get_encoder(pt->mime_type);
+		return _(desc->text);
+	}
+	return NULL;
+}
+
+
+/*this function makes a special case for speex/8000.
+This codec is variable bitrate. The 8kbit/s mode is interesting when having a low upload bandwidth, but its quality
+is not very good. We 'd better use its 15kbt/s mode when we have enough bandwidth*/
+static int get_codec_bitrate(LinphoneCore *lc, const PayloadType *pt){
+	int upload_bw=linphone_core_get_upload_bandwidth(lc);
+	if (bandwidth_is_greater(upload_bw,129) || (bandwidth_is_greater(upload_bw,33) && !linphone_core_video_enabled(lc)) ) {
+		if (strcmp(pt->mime_type,"speex")==0 && pt->clock_rate==8000){
+			return 15000;
+		}
+	}
 	return pt->normal_bitrate;
 }
-const char *payload_type_get_mime(PayloadType *pt){
-	return pt->mime_type;
-}
 
-int payload_type_get_rate(PayloadType *pt){
-	return pt->clock_rate;
-}
-
-static double get_audio_payload_bandwidth(const PayloadType *pt){
+static double get_audio_payload_bandwidth(LinphoneCore *lc, const PayloadType *pt){
 	double npacket=50;
 	double packet_size;
 	int bitrate;
-	bitrate=pt->normal_bitrate;
-	packet_size=(double)(bitrate/(50*8))+UDP_HDR_SZ+RTP_HDR_SZ+IP4_HDR_SZ;
+	bitrate=get_codec_bitrate(lc,pt);
+	packet_size= (((double)bitrate)/(50*8))+UDP_HDR_SZ+RTP_HDR_SZ+IP4_HDR_SZ;
 	return packet_size*8.0*npacket;
 }
 
 void linphone_core_update_allocated_audio_bandwidth_in_call(LinphoneCore *lc, const PayloadType *pt){
-	lc->audio_bw=(int)(get_audio_payload_bandwidth(pt)/1000.0);
+	lc->audio_bw=(int)(get_audio_payload_bandwidth(lc,pt)/1000.0);
 	/*update*/
 	linphone_core_set_download_bandwidth(lc,lc->net_conf.download_bw);
 	linphone_core_set_upload_bandwidth(lc,lc->net_conf.upload_bw);
@@ -209,8 +246,9 @@ void linphone_core_update_allocated_audio_bandwidth(LinphoneCore *lc){
 	for(elem=linphone_core_get_audio_codecs(lc);elem!=NULL;elem=elem->next){
 		PayloadType *pt=(PayloadType*)elem->data;
 		if (payload_type_enabled(pt)){
+			int pt_bitrate=get_codec_bitrate(lc,pt);
 			if (max==NULL) max=pt;
-			else if (max->normal_bitrate<pt->normal_bitrate){
+			else if (max->normal_bitrate<pt_bitrate){
 				max=pt;
 			}
 		}
@@ -241,8 +279,14 @@ bool_t linphone_core_check_payload_type_usability(LinphoneCore *lc, PayloadType
 	switch (pt->type){
 		case PAYLOAD_AUDIO_CONTINUOUS:
 		case PAYLOAD_AUDIO_PACKETIZED:
-			codec_band=get_audio_payload_bandwidth(pt);
+			codec_band=get_audio_payload_bandwidth(lc,pt);
 			ret=bandwidth_is_greater(min_audio_bw*1000,codec_band);
+			/*hack to avoid using uwb codecs when having low bitrate and video*/
+			if (bandwidth_is_greater(199,min_audio_bw)){
+				if (linphone_core_video_enabled(lc) && pt->clock_rate>16000){
+					ret=FALSE;
+				}
+			}
 			//ms_message("Payload %s: %g",pt->mime_type,codec_band);
 			break;
 		case PAYLOAD_VIDEO:
@@ -250,7 +294,7 @@ bool_t linphone_core_check_payload_type_usability(LinphoneCore *lc, PayloadType
 				/*let the video use all the bandwidth minus the maximum bandwidth used by audio */
 				if (min_video_bw>0)
 					pt->normal_bitrate=min_video_bw*1000;
-				else 
+				else
 					pt->normal_bitrate=1500000; /*around 1.5 Mbit/s*/
 				ret=TRUE;
 			}
@@ -258,159 +302,12 @@ bool_t linphone_core_check_payload_type_usability(LinphoneCore *lc, PayloadType
 			break;
 	}
 	/*if (!ret) ms_warning("Payload %s is not usable with your internet connection.",pt->mime_type);*/
-	
+
 	return ret;
 }
 
-static PayloadType * find_payload(RtpProfile *prof, PayloadType *pt /*from config*/){
-	PayloadType *candidate=NULL;
-	int i;
-	PayloadType *it;
-	for(i=0;i<127;++i){
-		it=rtp_profile_get_payload(prof,i);
-		if (it!=NULL && strcasecmp(pt->mime_type,it->mime_type)==0 
-			&& (pt->clock_rate==it->clock_rate || pt->clock_rate<=0)
-			&& payload_type_get_user_data(it)==NULL ){
-			if ( (pt->recv_fmtp && it->recv_fmtp && strcasecmp(pt->recv_fmtp,it->recv_fmtp)==0) ||
-				(pt->recv_fmtp==NULL && it->recv_fmtp==NULL) ){
-				/*exact match*/
-				return it;
-			}else candidate=it;
-		}
-	}
-	return candidate;
-}
-
-static bool_t check_h264_packmode(PayloadType *payload, MSFilterDesc *desc){
-	if (payload->recv_fmtp==NULL || strstr(payload->recv_fmtp,"packetization-mode")==0){
-		/*this is packetization-mode=0 H264, we only support it with a multislicing
-		enabled version of x264*/
-		if (strstr(desc->text,"x264") && strstr(desc->text,"multislicing")==0){
-			/*this is x264 without multisclicing*/
-			ms_message("Disabling packetization-mode=0 H264 codec because "
-			"of lack of multislicing support");
-			return FALSE;
-		}
-	}
-	return TRUE;
-}
-
-static MSList *fix_codec_list(RtpProfile *prof, MSList *conflist)
-{
-	MSList *elem;
-	MSList *newlist=NULL;
-	PayloadType *payload,*confpayload;
-	
-	for (elem=conflist;elem!=NULL;elem=ms_list_next(elem))
-	{
-		confpayload=(PayloadType*)elem->data;
-		payload=find_payload(prof,confpayload);
-		if (payload!=NULL){
-			if (ms_filter_codec_supported(confpayload->mime_type)){
-				MSFilterDesc *desc=ms_filter_get_encoder(confpayload->mime_type);
-				if (strcasecmp(confpayload->mime_type,"H264")==0){
-					if (!check_h264_packmode(confpayload,desc)){
-						continue;
-					}
-				}
-				payload_type_set_user_data(payload,(void*)desc->text);
-				payload_type_set_enable(payload,payload_type_enabled(confpayload));
-				newlist=ms_list_append(newlist,payload);
-			}
-		}
-		else{
-			ms_warning("Cannot support %s/%i: does not exist.",confpayload->mime_type,
-								confpayload->clock_rate);
-		}
-	}
-	return newlist;
-}
-
-
-void linphone_core_setup_local_rtp_profile(LinphoneCore *lc)
-{
-	int i;
-	MSList *audiopt,*videopt;
-	PayloadType *payload;
-	bool_t prepend;
-	lc->local_profile=rtp_profile_clone_full(&av_profile);
-	
-	/* first look at the list given by configuration file to see if 
-	it is correct */
-	audiopt=fix_codec_list(lc->local_profile,lc->codecs_conf.audio_codecs);
-	videopt=fix_codec_list(lc->local_profile,lc->codecs_conf.video_codecs);
-	
-	/* now find and add payloads that are not listed in the configuration
-	codec list */
-	for (i=0;i<127;i++)
-	{
-		payload=rtp_profile_get_payload(lc->local_profile,i);
-		if (payload!=NULL){
-			if (payload_type_get_user_data(payload)!=NULL) continue;
-			/* find a mediastreamer codec for this payload type */
-			if (ms_filter_codec_supported(payload->mime_type)){
-				MSFilterDesc *desc=ms_filter_get_encoder(payload->mime_type);
-				ms_message("Adding new codec %s/%i",payload->mime_type,payload->clock_rate);
-				payload_type_set_enable(payload,1);
-				payload_type_set_user_data(payload,(void *)desc->text);
-				prepend=FALSE;
-				/* by default, put speex, mpeg4, or h264 on top of list*/
-				if (strcmp(payload->mime_type,"speex")==0)
-					prepend=TRUE;
-				else if (strcmp(payload->mime_type,"MP4V-ES")==0)
-					prepend=TRUE;
-				else if (strcasecmp(payload->mime_type,"H264")==0){
-					if (check_h264_packmode(payload,desc))
-						prepend=TRUE;
-					else continue;
-				}
-				switch (payload->type){
-					case PAYLOAD_AUDIO_CONTINUOUS:
-					case PAYLOAD_AUDIO_PACKETIZED:
-							if (prepend)
-								audiopt=ms_list_prepend(audiopt,(void *)payload);	
-							else
-								audiopt=ms_list_append(audiopt,(void *)payload);
-						break;
-					case PAYLOAD_VIDEO:
-							if (prepend)
-								videopt=ms_list_prepend(videopt,(void *)payload);
-							else
-								videopt=ms_list_append(videopt,(void *)payload);
-						break;
-					default:
-						ms_error("Unsupported rtp media type.");
-				}
-			}
-		}
-	}
-	ms_list_for_each(lc->codecs_conf.audio_codecs,(void (*)(void*))payload_type_destroy);
-	ms_list_for_each(lc->codecs_conf.video_codecs,(void (*)(void *))payload_type_destroy);
-	ms_list_free(lc->codecs_conf.audio_codecs);
-	ms_list_free(lc->codecs_conf.video_codecs);
-	/* set the fixed lists instead:*/
-	lc->codecs_conf.audio_codecs=audiopt;
-	lc->codecs_conf.video_codecs=videopt;
-	linphone_core_update_allocated_audio_bandwidth(lc);
-}
-
-int from_2char_without_params(osip_from_t *from,char **str)
-{
-	osip_from_t *tmpfrom=NULL;
-	osip_from_clone(from,&tmpfrom);
-	if (tmpfrom!=NULL){
-		while(!osip_list_eol(&tmpfrom->gen_params,0)){
-			osip_generic_param_t *param=(osip_generic_param_t*)osip_list_get(&tmpfrom->gen_params,0);
-			osip_generic_param_free(param);
-			osip_list_remove(&tmpfrom->gen_params,0);
-		}
-	}else return -1;
-	osip_from_to_str(tmpfrom,str);
-	osip_from_free(tmpfrom);
-	return 0;
-}
-
 bool_t lp_spawn_command_line_sync(const char *command, char **result,int *command_ret){
+#if !defined(_WIN32_WCE)
 	FILE *f=popen(command,"r");
 	if (f!=NULL){
 		int err;
@@ -426,6 +323,7 @@ bool_t lp_spawn_command_line_sync(const char *command, char **result,int *comman
 		if (command_ret!=NULL) *command_ret=err;
 		return TRUE;
 	}
+#endif /*_WIN32_WCE*/
 	return FALSE;
 }
 
@@ -438,12 +336,12 @@ bool_t host_has_ipv6_network()
 	struct ifaddrs *ifp;
 	struct ifaddrs *ifpstart;
 	bool_t ipv6_present=FALSE;
-	
+
 	if (getifaddrs (&ifpstart) < 0)
 	{
 		return FALSE;
 	}
-	
+
 	for (ifp=ifpstart; ifp != NULL; ifp = ifp->ifa_next)
 	{
 		if (!ifp->ifa_addr)
@@ -451,7 +349,7 @@ bool_t host_has_ipv6_network()
 
 		switch (ifp->ifa_addr->sa_family) {
 		case AF_INET:
-		        
+
 			break;
 		case AF_INET6:
 		    ipv6_present=TRUE;
@@ -586,13 +484,18 @@ void linphone_core_run_stun_tests(LinphoneCore *lc, LinphoneCall *call){
 		bool_t got_audio,got_video;
 		bool_t cone_audio=FALSE,cone_video=FALSE;
 		struct timeval init,cur;
+		SalEndpointCandidate *ac,*vc;
+		
+		ac=&call->localdesc->streams[0].candidates[0];
+		vc=&call->localdesc->streams[1].candidates[0];
+		
 		if (parse_stun_server_addr(server,&ss,&ss_len)<0){
 			ms_error("Fail to parser stun server address: %s",server);
 			return;
 		}
 		if (lc->vtable.display_status!=NULL)
 			lc->vtable.display_status(lc,_("Stun lookup in progress..."));
-		
+
 		/*create the two audio and video RTP sockets, and send STUN message to our stun server */
 		sock1=create_socket(linphone_core_get_audio_port(lc));
 		if (sock1<0) return;
@@ -618,20 +521,20 @@ void linphone_core_run_stun_tests(LinphoneCore *lc, LinphoneCall *call){
 			usleep(10000);
 #endif
 
-			if (recvStunResponse(sock1,call->audio_params.natd_addr,
-						&call->audio_params.natd_port,&id)>0){
+			if (recvStunResponse(sock1,ac->addr,
+						&ac->port,&id)>0){
 				ms_message("STUN test result: local audio port maps to %s:%i",
-						call->audio_params.natd_addr,
-						call->audio_params.natd_port);
+						ac->addr,
+						ac->port);
 				if (id==11)
 					cone_audio=TRUE;
 				got_audio=TRUE;
 			}
-			if (recvStunResponse(sock2,call->video_params.natd_addr,
-							&call->video_params.natd_port,&id)>0){
+			if (recvStunResponse(sock2,vc->addr,
+							&vc->port,&id)>0){
 				ms_message("STUN test result: local video port maps to %s:%i",
-					call->video_params.natd_addr,
-					call->video_params.natd_port);
+					vc->addr,
+					vc->port);
 				if (id==22)
 					cone_video=TRUE;
 				got_video=TRUE;
@@ -645,7 +548,10 @@ void linphone_core_run_stun_tests(LinphoneCore *lc, LinphoneCall *call){
 		}else{
 			if (!cone_audio) {
 				ms_warning("NAT is symmetric for audio port");
-				call->audio_params.natd_port=0;
+				/*
+				ac->addr[0]='\0';
+				ac->port=0;
+				*/
 			}
 		}
 		if (sock2>=0){
@@ -654,10 +560,17 @@ void linphone_core_run_stun_tests(LinphoneCore *lc, LinphoneCall *call){
 			}else{
 				if (!cone_video) {
 					ms_warning("NAT is symmetric for video port.");
-					call->video_params.natd_port=0;
+					/*
+					vc->addr[0]='\0';
+					vc->port=0;
+					*/
 				}
 			}
 		}
+		if ((ac->addr[0]!='\0' && vc->addr[0]!='\0' && strcmp(ac->addr,vc->addr)==0)
+		    || sock2==-1){
+			strcpy(call->localdesc->addr,ac->addr);
+		}
 		close_socket(sock1);
 		if (sock2>=0) close_socket(sock2);
 	}
@@ -746,16 +659,50 @@ int linphone_core_wake_up_possible_already_running_instance(
 	return -1;
 }
 
-int linphone_core_get_local_ip_for(const char *dest, char *result){
+#ifdef HAVE_GETIFADDRS
+
+#include <ifaddrs.h>
+static int get_local_ip_with_getifaddrs(int type, char *address, int size)
+{
+	struct ifaddrs *ifp;
+	struct ifaddrs *ifpstart;
+	int ret = 0;
+
+	if (getifaddrs(&ifpstart) < 0) {
+		return -1;
+	}
+
+	for (ifp = ifpstart; ifp != NULL; ifp = ifp->ifa_next) {
+		if (ifp->ifa_addr && ifp->ifa_addr->sa_family == type
+			&& (ifp->ifa_flags & IFF_RUNNING) && !(ifp->ifa_flags & IFF_LOOPBACK))
+		{
+			getnameinfo(ifp->ifa_addr,
+						(type == AF_INET6) ?
+						sizeof(struct sockaddr_in6) : sizeof(struct sockaddr_in),
+						address, size, NULL, 0, NI_NUMERICHOST);
+			if (strchr(address, '%') == NULL) {	/*avoid ipv6 link-local addresses */
+				/*ms_message("getifaddrs() found %s",address);*/
+				ret++;
+			}
+		}
+	}
+	freeifaddrs(ifpstart);
+	return ret;
+}
+#endif
+
+
+static int get_local_ip_for_with_connect(int type, const char *dest, char *result){
 	int err,tmp;
 	struct addrinfo hints;
 	struct addrinfo *res=NULL;
 	struct sockaddr_storage addr;
+	struct sockaddr *p_addr=(struct sockaddr*)&addr;
 	ortp_socket_t sock;
 	socklen_t s;
 
 	memset(&hints,0,sizeof(hints));
-	hints.ai_family=PF_UNSPEC;
+	hints.ai_family=(type==AF_INET6) ? PF_INET6 : PF_INET;
 	hints.ai_socktype=SOCK_DGRAM;
 	/*hints.ai_flags=AI_NUMERICHOST|AI_CANONNAME;*/
 	err=getaddrinfo(dest,"5060",&hints,&res);
@@ -777,7 +724,7 @@ int linphone_core_get_local_ip_for(const char *dest, char *result){
 	if (err<0) {
 		ms_error("Error in connect: %s",strerror(errno));
  		freeaddrinfo(res);
- 		close(sock);
+ 		close_socket(sock);
 		return -1;
 	}
 	freeaddrinfo(res);
@@ -786,14 +733,45 @@ int linphone_core_get_local_ip_for(const char *dest, char *result){
 	err=getsockname(sock,(struct sockaddr*)&addr,&s);
 	if (err!=0) {
 		ms_error("Error in getsockname: %s",strerror(errno));
-		close(sock);
+		close_socket(sock);
 		return -1;
 	}
+	if (p_addr->sa_family==AF_INET){
+		struct sockaddr_in *p_sin=(struct sockaddr_in*)p_addr;
+		if (p_sin->sin_addr.s_addr==0){
+			close_socket(sock);
+			return -1;
+		}
+	}
 	err=getnameinfo((struct sockaddr *)&addr,s,result,LINPHONE_IPADDR_SIZE,NULL,0,NI_NUMERICHOST);
 	if (err!=0){
 		ms_error("getnameinfo error: %s",strerror(errno));
 	}
-	close(sock);
+	close_socket(sock);
 	ms_message("Local interface to reach %s is %s.",dest,result);
 	return 0;
 }
+
+int linphone_core_get_local_ip_for(int type, const char *dest, char *result){
+	if (dest==NULL) {
+		if (type==AF_INET)
+			dest="87.98.157.38"; /*a public IP address*/
+		else dest="2a00:1450:8002::68";
+	}
+	strcpy(result,type==AF_INET ? "127.0.0.1" : "::1");
+#ifdef HAVE_GETIFADDRS
+	{
+		int found_ifs;
+	
+		found_ifs=get_local_ip_with_getifaddrs(type,result,LINPHONE_IPADDR_SIZE);
+		if (found_ifs==1){
+			return 0;
+		}else if (found_ifs<=0){
+			/*absolutely no network on this machine */
+			return -1;
+		}
+	}
+#endif
+	/*else use connect to find the best local ip address */
+	return get_local_ip_for_with_connect(type,dest,result);
+}
