@@ -510,8 +510,11 @@ int sal_call_accept(SalOp * h){
 		if (h->sdp_offering) {
 			set_sdp_from_desc(msg,h->base.local_media);
 		}else{
-			if (h->sdp_answer)
+			if (h->sdp_answer){
 				set_sdp(msg,h->sdp_answer);
+				sdp_message_free(h->sdp_answer);
+				h->sdp_answer=NULL;
+			}
 		}
 	}else{
 		ms_error("You are accepting a call but not defined any media capabilities !");
@@ -760,7 +763,11 @@ static void handle_reinvite(Sal *sal,  eXosip_event_t *ev){
 		sdp_to_media_description(sdp,op->base.remote_media);
 		sdp_message_free(sdp);
 		sdp_process(op);
-		set_sdp(msg,op->sdp_answer);
+		if (op->sdp_answer!=NULL){
+			set_sdp(msg,op->sdp_answer);
+			sdp_message_free(op->sdp_answer);
+			op->sdp_answer=NULL;
+		}
 	}else {
 		op->sdp_offering=TRUE;
 		set_sdp_from_desc(msg,op->base.local_media);
@@ -872,8 +879,11 @@ static void call_accepted(Sal *sal, eXosip_event_t *ev){
 		_osip_list_set_empty(&msg->contacts,(void (*)(void*))osip_contact_free);
 		osip_message_set_contact(msg,contact);
 	}
-	if (op->sdp_answer)
-			set_sdp(msg,op->sdp_answer);
+	if (op->sdp_answer){
+		set_sdp(msg,op->sdp_answer);
+		sdp_message_free(op->sdp_answer);
+		op->sdp_answer=NULL;
+	}
 	eXosip_call_send_ack(ev->did,msg);
 	sal->callbacks.call_accepted(op);
 }
@@ -994,10 +1004,12 @@ static void authentication_ok(Sal *sal, eXosip_event_t *ev){
 static bool_t call_failure(Sal *sal, eXosip_event_t *ev){
 	SalOp *op;
 	int code=0;
+	char* computedReason=NULL;
 	const char *reason=NULL;
 	SalError error=SalErrorUnknown;
 	SalReason sr=SalReasonUnknown;
 	
+
 	op=(SalOp*)find_op(sal,ev);
 
 	if (op==NULL) {
@@ -1008,6 +1020,15 @@ static bool_t call_failure(Sal *sal, eXosip_event_t *ev){
 	if (ev->response){
 		code=osip_message_get_status_code(ev->response);
 		reason=osip_message_get_reason_phrase(ev->response);
+		osip_header_t *h=NULL;
+		if (!osip_message_header_get_byname(	ev->response
+											,"Reason"
+											,0
+											,&h)) {
+			computedReason = ms_strdup_printf("%s %s",reason,osip_header_get_value(h));
+			reason = computedReason;
+
+		}
 	}
 	switch(code)
 	{
@@ -1054,6 +1075,9 @@ static bool_t call_failure(Sal *sal, eXosip_event_t *ev){
 			}else error=SalErrorNoResponse;
 	}
 	sal->callbacks.call_failure(op,error,sr,reason);
+	if (computedReason != NULL){
+		ms_free(computedReason);
+	}
 	return TRUE;
 }
 
@@ -1575,8 +1599,6 @@ int sal_unregister(SalOp *h){
 	return 0;
 }
 
-
-
 SalAddress * sal_address_new(const char *uri){
 	osip_from_t *from;
 	osip_from_init(&from);
@@ -1584,6 +1606,11 @@ SalAddress * sal_address_new(const char *uri){
 		osip_from_free(from);
 		return NULL;
 	}
+	if (from->displayname!=NULL && from->displayname[0]=='"'){
+		char *unquoted=osip_strdup_without_quote(from->displayname);
+		osip_free(from->displayname);
+		from->displayname=unquoted;
+	}
 	return (SalAddress*)from;
 }
 
@@ -1621,8 +1648,9 @@ void sal_address_set_display_name(SalAddress *addr, const char *display_name){
 		osip_free(u->displayname);
 		u->displayname=NULL;
 	}
-	if (display_name!=NULL)
+	if (display_name!=NULL && display_name[0]!='\0'){
 		u->displayname=osip_strdup(display_name);
+	}
 }
 
 void sal_address_set_username(SalAddress *addr, const char *username){
@@ -1673,7 +1701,19 @@ void sal_address_clean(SalAddress *addr){
 
 char *sal_address_as_string(const SalAddress *u){
 	char *tmp,*ret;
-	osip_from_to_str((osip_from_t*)u,&tmp);
+	osip_from_t *from=(osip_from_t *)u;
+	char *old_displayname=NULL;
+	/* hack to force use of quotes around the displayname*/
+	if (from->displayname!=NULL
+	    && from->displayname[0]!='"'){
+		old_displayname=from->displayname;
+		from->displayname=osip_enquote(from->displayname);
+	}
+	osip_from_to_str(from,&tmp);
+	if (old_displayname!=NULL){
+		ms_free(from->displayname);
+		from->displayname=old_displayname;
+	}
 	ret=ms_strdup(tmp);
 	osip_free(tmp);
 	return ret;
@@ -1690,6 +1730,7 @@ char *sal_address_as_string_uri_only(const SalAddress *u){
 void sal_address_destroy(SalAddress *u){
 	osip_from_free((osip_from_t*)u);
 }
+
 void sal_set_keepalive_period(Sal *ctx,unsigned int value) {
 	eXosip_set_option (EXOSIP_OPT_UDP_KEEP_ALIVE, &value);
 }
