@@ -86,7 +86,16 @@ typedef enum MSFilterId{
 	MS_DSCAP_ID,
 	MS_AQ_READ_ID,
 	MS_AQ_WRITE_ID,
-	MS_EQUALIZER_ID
+	MS_EQUALIZER_ID,
+	MS_JPEG_DEC_ID,
+	MS_JPEG_ENC_ID,
+	MS_PULSE_READ_ID,
+	MS_PULSE_WRITE_ID,
+	MS_DRAWDIB_DISPLAY_ID,
+	MS_CHANNEL_ADAPTER_ID,
+	MS_AUDIO_MIXER_ID,
+	MS_ITC_SINK_ID,
+	MS_ITC_SOURCE_ID
 } MSFilterId;
 
 
