@@ -61,6 +61,8 @@ void audio_stream_free(AudioStream *stream)
 	if (stream->volsend!=NULL) ms_filter_destroy(stream->volsend);
 	if (stream->equalizer!=NULL) ms_filter_destroy(stream->equalizer);
 	if (stream->ticker!=NULL) ms_ticker_destroy(stream->ticker);
+	if (stream->read_resampler!=NULL) ms_filter_destroy(stream->read_resampler);
+	if (stream->write_resampler!=NULL) ms_filter_destroy(stream->write_resampler);
 	ms_free(stream);
 }
 
@@ -79,23 +81,11 @@ static void on_dtmf_received(RtpSession *s, int dtmf, void * user_data)
 	}
 }
 
-#if 0
-
-static void on_timestamp_jump(RtpSession *s,uint32_t* ts, void * user_data)
-{
-	ms_warning("The remote sip-phone has send data with a future timestamp: %u,"
-			"resynchronising session.",*ts);
-	rtp_session_reset(s);
-}
-
-#endif
-
-
 bool_t ms_is_ipv6(const char *remote){
 	bool_t ret=FALSE;
 #ifdef INET6
 	struct addrinfo hints, *res0;
-	
+
 	int err;
 	memset(&hints, 0, sizeof(hints));
 	hints.ai_family = PF_UNSPEC;
@@ -105,12 +95,21 @@ bool_t ms_is_ipv6(const char *remote){
 		ms_warning ("get_local_addr_for: %s", gai_strerror(err));
 		return FALSE;
 	}
-	ret=(res0->ai_addr->sa_family==AF_INET6); 
+	ret=(res0->ai_addr->sa_family==AF_INET6);
 	freeaddrinfo(res0);
 #endif
 	return ret;
 }
 
+static void audio_stream_configure_resampler(MSFilter *resampler,MSFilter *from,MSFilter *to) {
+	int from_rate=0, to_rate=0;
+	ms_filter_call_method(from,MS_FILTER_GET_SAMPLE_RATE,&from_rate);
+	ms_filter_call_method(to,MS_FILTER_GET_SAMPLE_RATE,&to_rate);
+	ms_filter_call_method(resampler,MS_FILTER_SET_SAMPLE_RATE,&from_rate);
+	ms_filter_call_method(resampler,MS_FILTER_SET_OUTPUT_SAMPLE_RATE,&to_rate);
+	ms_debug("configuring from rate[%i] to rate [%i]",from_rate,to_rate);
+}
+
 RtpSession * create_duplex_rtpsession( int locport, bool_t ipv6){
 	RtpSession *rtpr;
 	rtpr=rtp_session_new(RTP_SESSION_SENDRECV);
@@ -122,12 +121,13 @@ RtpSession * create_duplex_rtpsession( int locport, bool_t ipv6){
 	rtp_session_set_local_addr(rtpr,ipv6 ? "::" : "0.0.0.0",locport);
 	rtp_session_signal_connect(rtpr,"timestamp_jump",(RtpCallback)rtp_session_resync,(long)NULL);
 	rtp_session_signal_connect(rtpr,"ssrc_changed",(RtpCallback)rtp_session_resync,(long)NULL);
+	rtp_session_set_ssrc_changed_threshold(rtpr,0);
 	return rtpr;
 }
 
 #if defined(_WIN32_WCE)
 time_t
-time (time_t *t)
+ms_time (time_t *t)
 {
     DWORD timemillis = GetTickCount();
 	if (timemillis>0)
@@ -145,9 +145,9 @@ bool_t audio_stream_alive(AudioStream * stream, int timeout){
 	if (stats->recv!=0){
 		if (stats->recv!=stream->last_packet_count){
 			stream->last_packet_count=stats->recv;
-			stream->last_packet_time=time(NULL);
+			stream->last_packet_time=ms_time(NULL);
 		}else{
-			if (time(NULL)-stream->last_packet_time>timeout){
+			if (ms_time(NULL)-stream->last_packet_time>timeout){
 				/* more than timeout seconds of inactivity*/
 				return FALSE;
 			}
@@ -179,7 +179,7 @@ void audio_stream_change_decoder(AudioStream *stream, int payload){
 			ms_filter_link (stream->rtprecv, 0, stream->decoder, 0);
 			ms_filter_link (stream->decoder,0 , stream->dtmfgen, 0);
 			ms_filter_preprocess(stream->decoder,stream->ticker);
-			
+
 		}else{
 			ms_warning("No decoder found for %s",pt->mime_type);
 		}
@@ -208,22 +208,22 @@ int audio_stream_start_full(AudioStream *stream, RtpProfile *profile, const char
 	if (remport>0) rtp_session_set_remote_addr_full(rtps,remip,remport,rem_rtcp_port);
 	rtp_session_set_payload_type(rtps,payload);
 	rtp_session_set_jitter_compensation(rtps,jitt_comp);
-	
+
 	if (remport>0)
 		ms_filter_call_method(stream->rtpsend,MS_RTP_SEND_SET_SESSION,rtps);
 	stream->rtprecv=ms_filter_new(MS_RTP_RECV_ID);
 	ms_filter_call_method(stream->rtprecv,MS_RTP_RECV_SET_SESSION,rtps);
 	stream->session=rtps;
-	
+
 	stream->dtmfgen=ms_filter_new(MS_DTMF_GEN_ID);
 	rtp_session_signal_connect(rtps,"telephone-event",(RtpCallback)on_dtmf_received,(unsigned long)stream);
 	rtp_session_signal_connect(rtps,"payload_type_changed",(RtpCallback)payload_type_changed,(unsigned long)stream);
-	
+
 	/* creates the local part */
 	if (captcard!=NULL) stream->soundread=ms_snd_card_create_reader(captcard);
 	else {
 		stream->soundread=ms_filter_new(MS_FILE_PLAYER_ID);
-		stream->resampler=ms_filter_new(MS_RESAMPLE_ID);
+		stream->read_resampler=ms_filter_new(MS_RESAMPLE_ID);
 		if (infile!=NULL) audio_stream_play(stream,infile);
 	}
 	if (playcard!=NULL) stream->soundwrite=ms_snd_card_create_writer(playcard);
@@ -231,7 +231,7 @@ int audio_stream_start_full(AudioStream *stream, RtpProfile *profile, const char
 		stream->soundwrite=ms_filter_new(MS_FILE_REC_ID);
 		if (outfile!=NULL) audio_stream_record(stream,outfile);
 	}
-	
+
 	/* creates the couple of encoder/decoder */
 	pt=rtp_profile_get_payload(profile,payload);
 	if (pt==NULL){
@@ -245,7 +245,7 @@ int audio_stream_start_full(AudioStream *stream, RtpProfile *profile, const char
 		ms_error("mediastream.c: No decoder available for payload %i.",payload);
 		return -1;
 	}
-	
+
 	if (use_ec) {
 		stream->ec=ms_filter_new(MS_SPEEX_EC_ID);
 		ms_filter_call_method(stream->ec,MS_FILTER_SET_SAMPLE_RATE,&pt->clock_rate);
@@ -279,11 +279,20 @@ int audio_stream_start_full(AudioStream *stream, RtpProfile *profile, const char
 	}
 
 	/* give the sound filters some properties */
-	ms_filter_call_method(stream->soundread,MS_FILTER_SET_SAMPLE_RATE,&pt->clock_rate);
-	ms_filter_call_method(stream->soundwrite,MS_FILTER_SET_SAMPLE_RATE,&pt->clock_rate);
+	if (ms_filter_call_method(stream->soundread,MS_FILTER_SET_SAMPLE_RATE,&pt->clock_rate) != 0) {
+		/* need to add resampler*/
+		if (stream->read_resampler == NULL) stream->read_resampler=ms_filter_new(MS_RESAMPLE_ID);
+	}
+
+	if (ms_filter_call_method(stream->soundwrite,MS_FILTER_SET_SAMPLE_RATE,&pt->clock_rate) != 0) {
+		/* need to add resampler*/
+		if (stream->write_resampler == NULL) stream->write_resampler=ms_filter_new(MS_RESAMPLE_ID);
+	}
+
+
 	tmp=1;
 	ms_filter_call_method(stream->soundwrite,MS_FILTER_SET_NCHANNELS, &tmp);
-	
+
 	/* give the encoder/decoder some parameters*/
 	ms_filter_call_method(stream->encoder,MS_FILTER_SET_SAMPLE_RATE,&pt->clock_rate);
 	ms_message("Payload's bitrate is %i",pt->normal_bitrate);
@@ -292,23 +301,30 @@ int audio_stream_start_full(AudioStream *stream, RtpProfile *profile, const char
 		ms_filter_call_method(stream->encoder,MS_FILTER_SET_BITRATE,&pt->normal_bitrate);
 	}
 	ms_filter_call_method(stream->decoder,MS_FILTER_SET_SAMPLE_RATE,&pt->clock_rate);
-	
+
 	if (pt->send_fmtp!=NULL) ms_filter_call_method(stream->encoder,MS_FILTER_ADD_FMTP, (void*)pt->send_fmtp);
 	if (pt->recv_fmtp!=NULL) ms_filter_call_method(stream->decoder,MS_FILTER_ADD_FMTP,(void*)pt->recv_fmtp);
-	
+
 	/*create the equalizer*/
 	stream->equalizer=ms_filter_new(MS_EQUALIZER_ID);
 	tmp=stream->eq_active;
 	ms_filter_call_method(stream->equalizer,MS_EQUALIZER_SET_ACTIVE,&tmp);
+	/*configure resampler if needed*/
+	if (stream->read_resampler){
+		audio_stream_configure_resampler(stream->read_resampler,stream->soundread,stream->rtpsend);
+	}
 
+	if (stream->write_resampler){
+		audio_stream_configure_resampler(stream->write_resampler,stream->rtprecv,stream->soundwrite);
+	}
 	/* and then connect all */
 	/* tip: draw yourself the picture if you don't understand */
-	
+
 	/*sending graph*/
 	ms_connection_helper_start(&h);
 	ms_connection_helper_link(&h,stream->soundread,-1,0);
-	if (stream->resampler)
-		ms_connection_helper_link(&h,stream->resampler,0,0);
+	if (stream->read_resampler)
+		ms_connection_helper_link(&h,stream->read_resampler,0,0);
 	if (stream->ec)
 		ms_connection_helper_link(&h,stream->ec,1,1);
 	if (stream->volsend)
@@ -327,14 +343,16 @@ int audio_stream_start_full(AudioStream *stream, RtpProfile *profile, const char
 		ms_connection_helper_link(&h,stream->volrecv,0,0);
 	if (stream->ec)
 		ms_connection_helper_link(&h,stream->ec,0,0);
+	if (stream->write_resampler)
+		ms_connection_helper_link(&h,stream->write_resampler,0,0);
 	ms_connection_helper_link(&h,stream->soundwrite,0,-1);
-	
+
 	/* create ticker */
 	stream->ticker=ms_ticker_new();
 	ms_ticker_set_name(stream->ticker,"Audio MSTicker");
 	ms_ticker_attach(stream->ticker,stream->soundread);
 	ms_ticker_attach(stream->ticker,stream->rtprecv);
-	
+
 	return 0;
 }
 
@@ -385,14 +403,10 @@ void audio_stream_set_rtcp_information(AudioStream *st, const char *cname, const
 
 void audio_stream_play(AudioStream *st, const char *name){
 	if (ms_filter_get_id(st->soundread)==MS_FILE_PLAYER_ID){
-		int from_rate=0, to_rate=0;
 		ms_filter_call_method_noarg(st->soundread,MS_FILE_PLAYER_CLOSE);
 		ms_filter_call_method(st->soundread,MS_FILE_PLAYER_OPEN,(void*)name);
-		ms_filter_call_method(st->soundread,MS_FILTER_GET_SAMPLE_RATE,&from_rate);
-		ms_filter_call_method(st->rtpsend,MS_FILTER_GET_SAMPLE_RATE,&to_rate);
-		if (st->resampler){
-			ms_filter_call_method(st->resampler,MS_FILTER_SET_SAMPLE_RATE,&from_rate);
-			ms_filter_call_method(st->resampler,MS_FILTER_SET_OUTPUT_SAMPLE_RATE,&to_rate);
+		if (st->read_resampler){
+			audio_stream_configure_resampler(st->read_resampler,st->soundread,st->rtpsend);
 		}
 		ms_filter_call_method_noarg(st->soundread,MS_FILE_PLAYER_START);
 	}else{
@@ -437,7 +451,7 @@ void audio_stream_set_relay_session_id(AudioStream *stream, const char *id){
 	ms_filter_call_method(stream->rtpsend, MS_RTP_SEND_SET_RELAY_SESSION_ID,(void*)id);
 }
 
-void audio_stream_set_echo_canceler_params(AudioStream *st, int tail_len_ms, int delay_ms, int framesize){
+void audio_stream_set_echo_canceller_params(AudioStream *st, int tail_len_ms, int delay_ms, int framesize){
 	st->ec_tail_len=tail_len_ms;
 	st->ec_delay=delay_ms;
 	st->ec_framesize=framesize;
@@ -490,14 +504,14 @@ void audio_stream_stop(AudioStream * stream)
 		MSConnectionHelper h;
 		ms_ticker_detach(stream->ticker,stream->soundread);
 		ms_ticker_detach(stream->ticker,stream->rtprecv);
-		
+
 		rtp_stats_display(rtp_session_get_stats(stream->session),"Audio session's RTP statistics");
-		
+
 		/*dismantle the outgoing graph*/
 		ms_connection_helper_start(&h);
 		ms_connection_helper_unlink(&h,stream->soundread,-1,0);
-		if (stream->resampler!=NULL)
-			ms_connection_helper_unlink(&h,stream->resampler,0,0);
+		if (stream->read_resampler!=NULL)
+			ms_connection_helper_unlink(&h,stream->read_resampler,0,0);
 		if (stream->ec!=NULL)
 			ms_connection_helper_unlink(&h,stream->ec,1,1);
 		if (stream->volsend!=NULL)
@@ -516,6 +530,8 @@ void audio_stream_stop(AudioStream * stream)
 			ms_connection_helper_unlink(&h,stream->volrecv,0,0);
 		if (stream->ec!=NULL)
 			ms_connection_helper_unlink(&h,stream->ec,0,0);
+		if (stream->write_resampler!=NULL)
+			ms_connection_helper_unlink(&h,stream->write_resampler,0,0);
 		ms_connection_helper_unlink(&h,stream->soundwrite,0,-1);
 
 	}
@@ -560,6 +576,10 @@ void ring_stop(RingStream *stream){
 	ms_filter_destroy(stream->source);
 	ms_filter_destroy(stream->sndwrite);
 	ms_free(stream);
+#ifdef _WIN32_WCE
+	ms_warning("Sleeping a bit after closing the audio device...");
+	ms_sleep(1);
+#endif
 }
 
 
@@ -571,3 +591,10 @@ int audio_stream_send_dtmf(AudioStream *stream, char dtmf)
 		ms_filter_call_method(stream->dtmfgen,MS_DTMF_GEN_PUT,&dtmf);
 	return 0;
 }
+
+void audio_stream_get_local_rtp_stats(AudioStream *stream, rtp_stats_t *lstats){
+	if (stream->session){
+		const rtp_stats_t *stats=rtp_session_get_stats(stream->session);
+		memcpy(lstats,stats,sizeof(*stats));
+	}else memset(lstats,0,sizeof(rtp_stats_t));
+}
