@@ -57,6 +57,7 @@ typedef struct Volume{
 	int ng_cut_time; /*noise gate cut time, after last speech detected*/
 	int ng_noise_dur;
 	float ng_threshold;
+	float ng_floorgain;
 	MSBufferizer *buffer;
 	bool_t ea_active;
 	bool_t agc_enabled;
@@ -84,6 +85,7 @@ static void volume_init(MSFilter *f){
 	v->ng_cut_time=100;/*milliseconds*/
 	v->ng_noise_dur=0;
 	v->ng_threshold=noise_thres;
+	v->ng_floorgain=0;
 #ifdef HAVE_SPEEXDSP
 	v->speex_pp=NULL;
 #endif
@@ -103,7 +105,7 @@ static void volume_uninit(MSFilter *f){
 static int volume_get(MSFilter *f, void *arg){
 	float *farg=(float*)arg;
 	Volume *v=(Volume*)f->data;
-	*farg=10*log10f((v->energy+1)/max_e);
+	*farg=10*ortp_log10f((v->energy+1)/max_e);
 	return 0;
 }
 
@@ -138,10 +140,10 @@ static inline float compute_gain(float static_gain, float energy, float weight){
 }
 
 /*
-The principle of this algorithm is that we apply a gain to the input signal which is opposite to the 
+The principle of this algorithm is that we apply a gain to the input signal which is opposite to the
 energy measured by the peer MSVolume.
 For example if some noise is played by the speaker, then the signal captured by the microphone will be lowered.
-The gain changes smoothly when the peer energy is decreasing, but is immediately changed when the peer energy is 
+The gain changes smoothly when the peer energy is decreasing, but is immediately changed when the peer energy is
 increasing.
 */
 
@@ -188,7 +190,7 @@ static void volume_noise_gate_process(Volume *v , float energy, mblk_t *om){
 	if ((energy/max_e)<v->ng_threshold){
 		v->ng_noise_dur+=(nsamples*1000)/v->sample_rate;
 		if (v->ng_noise_dur>v->ng_cut_time){
-			v->target_gain=0;
+			v->target_gain=v->ng_floorgain;
 		}
 	}else{
 		v->ng_noise_dur=0;
@@ -196,6 +198,14 @@ static void volume_noise_gate_process(Volume *v , float energy, mblk_t *om){
 	}
 }
 
+static int volume_set_db_gain(MSFilter *f, void *gain){
+	float *fgain=(float*)gain;
+	Volume *v=(Volume*)f->data;
+	v->gain=v->static_gain=v->target_gain = pow(10,(*fgain)/10);
+	ms_message("MSVolume set gain to [%f db], [%f] linear",*fgain,v->gain);
+	return 0;
+}
+
 static int volume_set_gain(MSFilter *f, void *arg){
 	float *farg=(float*)arg;
 	Volume *v=(Volume*)f->data;
@@ -203,6 +213,12 @@ static int volume_set_gain(MSFilter *f, void *arg){
 	return 0;
 }
 
+static int volume_get_gain(MSFilter *f, void *arg){
+	float *farg=(float*)arg;
+	Volume *v=(Volume*)f->data;
+	*farg = v->gain;
+	return 0;
+}
 
 static int volume_get_ea_state(MSFilter *f, void *arg){
 	int *barg=(int*)arg;
@@ -271,6 +287,12 @@ static int volume_set_noise_gate_threshold(MSFilter *f, void *arg){
 	return 0;
 }
 
+static int volume_set_noise_gate_floorgain(MSFilter *f, void *arg){
+	Volume *v=(Volume*)f->data;
+	v->ng_floorgain=*(float*)arg;
+	return 0;
+}
+
 static inline int16_t saturate(float val){
 	return (val>32767) ? 32767 : ( (val<-32767) ? -32767 : val);
 }
@@ -340,7 +362,7 @@ static void volume_process(MSFilter *f){
 			om->b_wptr+=nbytes;
 			en=update_energy((int16_t*)om->b_rptr,v->nsamples,en);
 			volume_agc_process(v,om);
-	
+
 			if (v->peer){
 				volume_echo_avoider_process(v,f->ticker->time);
 			}else v->target_gain=v->static_gain;
@@ -355,7 +377,7 @@ static void volume_process(MSFilter *f){
 		while((m=ms_queue_get(f->inputs[0]))!=NULL){
 			en=update_energy((int16_t*)m->b_rptr,(m->b_wptr-m->b_rptr)/2,en);
 			if (v->peer){
-				volume_echo_avoider_process(v,f->ticker->time);	
+				volume_echo_avoider_process(v,f->ticker->time);
 			}else v->target_gain=v->static_gain;
 
 			if (v->noise_gate_enabled)
@@ -381,6 +403,9 @@ static MSFilterMethod methods[]={
 	{	MS_VOLUME_ENABLE_AGC	,	volume_set_agc		},
 	{	MS_VOLUME_ENABLE_NOISE_GATE,	volume_enable_noise_gate},
 	{	MS_VOLUME_SET_NOISE_GATE_THRESHOLD,	volume_set_noise_gate_threshold},
+	{	MS_VOLUME_SET_NOISE_GATE_FLOORGAIN,	volume_set_noise_gate_floorgain},
+	{	MS_VOLUME_SET_DB_GAIN	,	volume_set_db_gain		},
+	{	MS_VOLUME_GET_GAIN	,	volume_get_gain		},
 	{	0			,	NULL			}
 };
 
