@@ -47,6 +47,8 @@ void video_stream_free (VideoStream * stream)
 		ms_filter_destroy (stream->source);
 	if (stream->output != NULL)
 		ms_filter_destroy (stream->output);
+	if (stream->encoder != NULL)
+		ms_filter_destroy (stream->encoder);
 	if (stream->decoder != NULL)
 		ms_filter_destroy (stream->decoder);
 	if (stream->sizeconv != NULL)
@@ -137,11 +139,11 @@ static void video_steam_process_rtcp(VideoStream *stream, mblk_t *m){
 }
 
 void video_stream_iterate(VideoStream *stream){
-	
+	/*
 	if (stream->output!=NULL)
 		ms_filter_call_method_noarg(stream->output,
 			MS_VIDEO_OUT_HANDLE_RESIZING);
-	
+	*/
 	if (stream->evq){
 		OrtpEvent *ev=ortp_ev_queue_get(stream->evq);
 		if (ev!=NULL){
@@ -172,7 +174,7 @@ VideoStream *video_stream_new(int locport, bool_t use_ipv6){
 }
 
 void video_stream_set_sent_video_size(VideoStream *stream, MSVideoSize vsize){
-  ms_message("Setting vidoe size %dx%d", vsize.width, vsize.height);
+  ms_message("Setting video size %dx%d", vsize.width, vsize.height);
 	stream->sent_vsize=vsize;
 }
 
@@ -184,7 +186,7 @@ void video_stream_enable_self_view(VideoStream *stream, bool_t val){
 	MSFilter *out=stream->output;
 	stream->corner=val ? 0 : -1;
 	if (out){
-		ms_filter_call_method(out,MS_VIDEO_OUT_SET_CORNER,&stream->corner);
+		ms_filter_call_method(out,MS_VIDEO_DISPLAY_SET_LOCAL_VIEW_MODE,&stream->corner);
 	}
 }
 
@@ -244,7 +246,11 @@ int video_stream_start (VideoStream *stream, RtpProfile *profile, const char *re
 	/* creates the filters */
 	stream->source = ms_web_cam_create_reader(cam);
 	stream->tee = ms_filter_new(MS_TEE_ID);
+#ifndef WIN32
 	stream->output=ms_filter_new(MS_VIDEO_OUT_ID);
+#else
+	stream->output=ms_filter_new(MS_DRAWDIB_DISPLAY_ID);
+#endif
 	stream->sizeconv=ms_filter_new(MS_SIZE_CONV_ID);
 	
 	if (pt->normal_bitrate>0){
@@ -262,7 +268,8 @@ int video_stream_start (VideoStream *stream, RtpProfile *profile, const char *re
 	ms_filter_call_method(stream->encoder,MS_FILTER_GET_FPS,&fps);
 	ms_message("Setting vsize=%ix%i, fps=%f",vsize.width,vsize.height,fps);
 	/* configure the filters */
-	ms_filter_call_method(stream->source,MS_FILTER_SET_FPS,&fps);
+	if (ms_filter_get_id(stream->source)!=MS_STATIC_IMAGE_ID)
+		ms_filter_call_method(stream->source,MS_FILTER_SET_FPS,&fps);
 	ms_filter_call_method(stream->source,MS_FILTER_SET_VIDEO_SIZE,&vsize);
 
 	/* get the output format for webcam reader */
@@ -289,9 +296,9 @@ int video_stream_start (VideoStream *stream, RtpProfile *profile, const char *re
 	disp_size.height=MS_VIDEO_SIZE_CIF_H;
 	tmp=1;
 	ms_filter_call_method(stream->output,MS_FILTER_SET_VIDEO_SIZE,&disp_size);
-	ms_filter_call_method(stream->output,MS_VIDEO_OUT_AUTO_FIT,&tmp);
+	ms_filter_call_method(stream->output,MS_VIDEO_DISPLAY_ENABLE_AUTOFIT,&tmp);
 	ms_filter_call_method(stream->output,MS_FILTER_SET_PIX_FMT,&format);
-	ms_filter_call_method(stream->output,MS_VIDEO_OUT_SET_CORNER,&stream->corner);
+	ms_filter_call_method(stream->output,MS_VIDEO_DISPLAY_SET_LOCAL_VIEW_MODE,&stream->corner);
 
 	if (pt->recv_fmtp!=NULL)
 		ms_filter_call_method(stream->decoder,MS_FILTER_ADD_FMTP,(void*)pt->recv_fmtp);
@@ -352,7 +359,7 @@ void video_stream_set_rtcp_information(VideoStream *st, const char *cname, const
 unsigned long video_stream_get_native_window_id(VideoStream *stream){
 	unsigned long id;
 	if (stream->output){
-		if (ms_filter_call_method(stream->output,MS_VIDEO_OUT_GET_NATIVE_WINDOW_ID,&id)==0)
+		if (ms_filter_call_method(stream->output,MS_VIDEO_DISPLAY_GET_NATIVE_WINDOW_ID,&id)==0)
 			return id;
 	}
 	return 0;
@@ -365,15 +372,21 @@ VideoStream * video_preview_start(MSWebCam *device, MSVideoSize disp_size){
 	MSPixFmt format;
 	float fps=(float)29.97;
 	int mirroring=1;
+	int corner=-1;
 
 	/* creates the filters */
 	stream->source = ms_web_cam_create_reader(device);
+#ifndef WIN32
 	stream->output = ms_filter_new(MS_VIDEO_OUT_ID);
+#else
+	stream->output = ms_filter_new(MS_DRAWDIB_DISPLAY_ID);
+#endif
 
 
 	/* configure the filters */
 	ms_filter_call_method(stream->source,MS_FILTER_SET_VIDEO_SIZE,&vsize);
-	ms_filter_call_method(stream->source,MS_FILTER_SET_FPS,&fps);
+	if (ms_filter_get_id(stream->source)!=MS_STATIC_IMAGE_ID)
+		ms_filter_call_method(stream->source,MS_FILTER_SET_FPS,&fps);
 	ms_filter_call_method(stream->source,MS_FILTER_GET_PIX_FMT,&format);
 	ms_filter_call_method(stream->source,MS_FILTER_GET_VIDEO_SIZE,&vsize);
 	if (format==MS_MJPEG){
@@ -387,7 +400,8 @@ VideoStream * video_preview_start(MSWebCam *device, MSVideoSize disp_size){
 	format=MS_YUV420P;
 	ms_filter_call_method(stream->output,MS_FILTER_SET_PIX_FMT,&format);
 	ms_filter_call_method(stream->output,MS_FILTER_SET_VIDEO_SIZE,&disp_size);
-	ms_filter_call_method(stream->output,MS_VIDEO_OUT_ENABLE_MIRRORING,&mirroring);
+	ms_filter_call_method(stream->output,MS_VIDEO_DISPLAY_ENABLE_MIRRORING,&mirroring);
+	ms_filter_call_method(stream->output,MS_VIDEO_DISPLAY_SET_LOCAL_VIEW_MODE,&corner);
 	/* and then connect all */
 
 	ms_filter_link(stream->source,0, stream->pixconv,0);
@@ -542,8 +556,11 @@ int video_stream_recv_only_start (VideoStream *stream, RtpProfile *profile, cons
 		ms_error("videostream.c: No codecs available for payload %i:%s.",payload,pt->mime_type);
 		return -1;
 	}
+#ifndef WIN32
 	stream->output=ms_filter_new(MS_VIDEO_OUT_ID);
-
+#else
+	stream->output=ms_filter_new(MS_DRAWDIB_DISPLAY_ID);
+#endif
 	/*force the decoder to output YUV420P */
 	format=MS_YUV420P;
 	/*ask the size-converter to always output CIF */
